#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Exploration des logs
# 
# Traitements de logs générés par un QCM.

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# In[2]:


get_ipython().run_line_magic('matplotlib', 'inline')


# ## Observations brutes

# In[3]:


import os
names = [os.path.join("logs", _) for _ in os.listdir("logs") if '.log' in _]
names = names[:1]
names


# In[4]:


with open(names[0], 'r', encoding="utf-8") as f:
    lines = f.readlines()
lines[5:10]


# In[5]:


from mathenjeu.datalog import enumerate_qcmlog
obs = list(enumerate_qcmlog(names))
obs[:5]


# In[6]:


import pandas
df = pandas.DataFrame(obs)
df.shape


# In[7]:


df.head().T


# In[8]:


set(df.alias)


# ## Préparation des données

# In[9]:


df2 = df[df.qtime == 'end'].copy()
df2.head().T


# In[10]:


cols = ['alias'] + [c for c in df2.columns if "simple_french_qcm" in c and '-a' in c]
df_question = df2[cols]
cols = ['alias'] + [c for c in df2.columns if "simple_french_qcm" in c and '-b' in c]
df_bouton = df2[cols]
cols = ['alias'] + [c for c in df2.columns if "simple_french_qcm" in c and '-nb' in c]
df_visit = df2[cols]
cols = ['alias'] + [c for c in df2.columns if "simple_french_qcm" in c and '-ANS' in c]
df_ans = df2[cols]
cols = ['alias'] + [c for c in df2.columns if "simple_french_qcm" in c and '-dur' in c]
df_dur = df2[cols]


# In[11]:


df_dur.head().T


# In[12]:


import numpy 

def aggnotnan_serie(values):
    res = []
    for v in values:
        if isinstance(v, float) and numpy.isnan(v):
            continue
        if pandas.isnull(v):
            continue
        if v in ('ok', 'on'):
            v = 1
        elif v == 'skip':
            v = 1000
        res.append(v)
    if len(res) > 0:        
        if isinstance(res[0], str):
            r = ",".join(str(_) for _ in res)
        else:
            if len(res) == 1:
                r = res[0]
            else:
                try:
                    r = sum(res)
                except:
                    r = 0
    else:
        r = numpy.nan
    return r


def aggnotnan(values):
    if isinstance(values, pandas.core.series.Series):
        r = aggnotnan_serie(values)
        return r
    else:
        res = []
        for col in values.columns:
            val = list(values[col])
            res.append(aggnotnan_serie(val))
        df = pandas.DataFrame(res, columns)
        return df
    
gr_ans = df_ans.groupby("alias").agg(aggnotnan)
gr_ans


# In[13]:


gr_dur = df_dur.groupby("alias").agg(aggnotnan)
gr_dur


# In[14]:


gr_dur.T.plot();


# In[15]:


gr_question = df_question.groupby("alias").agg(aggnotnan)
gr_question.T


# In[16]:


gr_bouton = df_bouton.groupby("alias").agg(aggnotnan)
gr_bouton.T


# In[17]:


gr_visit = df_visit.groupby("alias").agg(aggnotnan)
gr_visit.T


# ## Histogrammes

# In[18]:


nonan_question = gr_question.fillna(0)


# In[19]:


import matplotlib.pyplot as plt

fig, ax = plt.subplots(nonan_question.shape[0], 1, 
                       figsize=(8, nonan_question.shape[0]))
for i in range(0, nonan_question.shape[0]):
    ax[i].set_ylabel(nonan_question.index[i])
    ax[i].bar(list(range(nonan_question.shape[1])), 
              nonan_question.iloc[i,:])


# ## Clustering

# In[20]:


nonan_question = gr_question.fillna(0)


# In[21]:


from sklearn.cluster import KMeans
km = KMeans(n_clusters=2)


# In[22]:


km.fit(nonan_question)


# In[23]:


pred = km.predict(nonan_question)
pred


# In[24]:


solution = pandas.DataFrame(data=pred, columns=["cluster"], index=nonan_question.index)


# In[25]:


solution


# ## ACP

# In[26]:


nonan_question.shape


# In[27]:


from sklearn.decomposition import PCA
acp = PCA(n_components=2, svd_solver='arpack')
acp.fit(nonan_question)


# In[28]:


coord = acp.transform(nonan_question)
data = pandas.DataFrame(data=coord, columns=['X1', 'X2'], index=nonan_question.index)
data["cluster"] = solution
data


# In[29]:


import matplotlib.pyplot as plt
fig, ax = plt.subplots(1, 1)
colors = ['red', 'blue', 'orange', 'green']
for i in range(0, 2):
    data[data.cluster==i].plot(x="X1", y="X2", 
                               kind="scatter", 
                               ax=ax, label="c%d" % i, 
                               color=colors[i])


# In[30]:


data.columns


# In[31]:


data.loc[data.cluster == 0, 'X1'].values, data.loc[data.cluster == 0, 'X2'].values


# In[32]:


data["weight"] = 10
# Plot miles per gallon against horsepower with other semantics
fig, ax = plt.subplots(1, 1)
ax.plot(data.loc[data.cluster == 0, 'X1'].values, 
        data.loc[data.cluster == 0, 'X2'].values, 'o', label='c0')
ax.plot(data.loc[data.cluster == 1, 'X1'].values, 
        data.loc[data.cluster == 1, 'X2'].values, 'o', label='c1')
ind = list(data.index)
for i in range(0, data.shape[0]):
    ax.text(data.iloc[i, 0], data.iloc[i, 1], ind[i])
ax.set_title('Clustering représenté en 2 dimensions');


# ## Prediction
# 

# In[33]:


nonan_question.T


# In[34]:


xcols = nonan_question.columns[:15]
ycol = nonan_question.columns[16]
ycol, set(nonan_question[ycol])


# In[35]:


from sklearn.model_selection import train_test_split
X_train, X_test, y_train, y_test = train_test_split(nonan_question[xcols], nonan_question[ycol], test_size=0.5)


# In[36]:


from sklearn.ensemble import RandomForestClassifier
clr = RandomForestClassifier()
clr.fit(X_train, y_train)


# In[37]:


from sklearn.metrics import confusion_matrix
confusion_matrix(y_test, clr.predict(X_test))


# In[38]:





# In[39]: